from unittest.mock import call, patch

from django.core.cache import cache
from django.test import TestCase, override_settings
from fred_types import ContactHistoryId, ContactId
from regal import Contact
from regal.exceptions import ContactDoesNotExist

from ferda_verification.utils import get_contact_last_info


@override_settings(
    CACHES={"default": {"BACKEND": "django.core.cache.backends.locmem.LocMemCache"}}
)
class GetContactLastInfoTest(TestCase):
    def setUp(self):
        cache.clear()

    @patch("ferda_verification.utils.CONTACT", autospec=True)
    async def test_get_contact_last_info(self, contact_client):
        contacts = [
            Contact(contact_id=ContactId("c:1"), contact_handle="RIMMER"),
            Contact(contact_id=ContactId("c:2"), contact_handle="LISTER"),
            Contact(contact_id=ContactId("c:1"), contact_handle="RIMMER"),
        ]
        contact_client.get_contact_info.side_effect = contacts

        self.assertEqual(await get_contact_last_info(ContactId("c:1")), contacts[0])
        self.assertEqual(await get_contact_last_info(ContactId("c:2")), contacts[1])
        self.assertEqual(await get_contact_last_info(ContactId("c:1")), contacts[2])

        self.assertEqual(
            contact_client.mock_calls,
            [
                call.get_contact_info(ContactId("c:1")),
                call.get_contact_info(ContactId("c:2")),
                # no third call, since "c:1" is already cached
            ],
        )

    @patch("ferda_verification.utils.CONTACT", autospec=True)
    async def test_get_contact_last_info_does_not_exist(self, contact_client):
        contacts = [
            ContactDoesNotExist,
            Contact(contact_id=ContactId("c:1"), contact_handle="RIMMER"),
        ]
        contact_client.get_contact_info.side_effect = contacts
        contact_client.get_contact_history.return_value = {
            "timeline": [{"contact_history_id": "ch:1"}],
        }

        self.assertEqual(await get_contact_last_info(ContactId("c:1")), contacts[1])

        self.assertEqual(
            contact_client.mock_calls,
            [
                call.get_contact_info(ContactId("c:1")),
                call.get_contact_history(ContactId("c:1")),
                call.get_contact_info(ContactId("c:1"), history_id=ContactHistoryId("ch:1")),
            ],
        )
